// Vertex Shader  file "lambert.vert"

#version 330 core

uniform int hasBones;

uniform mat4 mv;
uniform mat4 perspective;
uniform mat4 normalMatrix;

uniform mat4 boneTransform[50];
uniform mat4 boneNormalTransform[50];

layout(location=1) in  vec3 position;
layout(location=2) in  vec3 normal;
layout(location=3) in  vec3 tangent;
layout(location=4) in  vec3 bitangent;
layout(location=5) in  vec2 tex_coord;

layout(location=10) in  ivec2 bone_influence;
layout(location=11) in  vec2 bone_weight;

out vec2 ex_coord;
out vec3 mvTangent;
out vec3 mvBitangent;
out vec4 mvPosition;
out vec3 mvNormal;
out mat3 TBN;


void main(void)
{
	vec4 p;
	vec4 n;
	vec4 t;
	vec4 b;

	if (hasBones==1)
	{
		vec4 p1 = (boneTransform[bone_influence.x] * vec4(position,1.0)) * bone_weight.x;
		p = p1+(boneTransform[bone_influence.y] * vec4(position,1.0)) * bone_weight.y;

		vec4 n1 = (boneNormalTransform[bone_influence.x] * vec4(normal,1.0)) * bone_weight.x;
		n = n1+(boneNormalTransform[bone_influence.y] * vec4(normal,1.0)) * bone_weight.y;

		vec4 t1 = (boneNormalTransform[bone_influence.x] * vec4(tangent,1.0)) * bone_weight.x;
		t = t1+(boneNormalTransform[bone_influence.y] * vec4(tangent,1.0)) * bone_weight.y;

		vec4 b1 = (boneNormalTransform[bone_influence.x] * vec4(bitangent,1.0)) * bone_weight.x;
		b = b1+(boneNormalTransform[bone_influence.y] * vec4(bitangent,1.0)) * bone_weight.y;
	} else {
		p = vec4(position,1.0);
		n = vec4(normal,1.0);
		t = vec4(tangent,1.0);
		//t = normalize(t - n * dot(n, t));
		b = vec4(bitangent,1.0);
	}

	mvPosition = mv * p;
	gl_Position = perspective * mvPosition;

	mvNormal = normalize((normalMatrix * n).xyz);
	mvTangent = normalize((normalMatrix * t).xyz);
	mvBitangent = normalize((normalMatrix * b).xyz);

	TBN = transpose(mat3(mvTangent, mvBitangent, mvNormal));

	ex_coord = tex_coord;

}